import 'dart:async';
import 'package:flutter/services.dart';
import 'package:flutter/material.dart';

class CHCycleView extends StatefulWidget {
  List<Map> imageData;
  double cycleWidth;
  double cycleHeight;
  Color unSelectetColor;
  Color selectetedColor;
  double pageControllerSize;
  int duration;
  final callBack;

  CHCycleView({Key key,
    this.imageData,
    this.cycleWidth,
    this.cycleHeight,
    this.unSelectetColor,
    this.selectetedColor,
    this.pageControllerSize,
    this.duration,
    this.callBack}) : super(key : key);

  createState() => CHCycleViewState(
      imageData,
      cycleWidth,
      cycleHeight,
      unSelectetColor,
      selectetedColor,
      pageControllerSize,
      duration,
      callBack);
}

const timeout = const Duration(seconds: 2);

class CHCycleViewState extends State<CHCycleView> with SingleTickerProviderStateMixin{
  List<Map> imageData;
  double cycleWidth;
  double cycleHeight;
  Color unSelectetColor;
  Color selectetedColor;
  double pageControllerSize;
  int duration;
  final callBack;
  bool isEndScroll = true;

  CHCycleViewState(
      this.imageData,
      this.cycleWidth,
      this.cycleHeight,
      this.unSelectetColor,
      this.selectetedColor,
      this.pageControllerSize,
      this.duration,
      this.callBack
      );

  PageController _pageController = PageController(initialPage: 1);
  TabController _tabController;
  List<String> _adPictures = [];
  List<Map> _tempShowPictures = [];
  Timer tempTimer;
  int _index = 1;

  @override
  void dispose() {
    // TODO: implement dispose
    _pageController.dispose();
    _tabController.dispose();
    super.dispose();
  }

  @override
  void initState() {
    _tabController = TabController(length: this.imageData.length, vsync: this);
    tempTimer = Timer.periodic(Duration(seconds: this.duration), _handleTimeout);
    for (var item in imageData) {
      _adPictures.add(item['imageUrl'].toString());
    }

    _tempShowPictures.add(imageData[imageData.length - 1]);
    for (var item in imageData) {
      _tempShowPictures.add(item);
    }
    _tempShowPictures.add(imageData[0]);


    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    // TODO: implement build
    return Container(
      alignment: Alignment.topLeft,
      width: this.cycleWidth,
      height: this.cycleHeight,
      child: Stack(
        children: <Widget>[
          NotificationListener(
            child: PageView(
              children: _getImagesWithUrl(),
              onPageChanged: _onPageChanged,
              controller: _pageController,
            ),
            onNotification: (ScrollNotification sn) {
              if (sn is ScrollEndNotification || sn is UserScrollNotification || _index == 1) {
                this.isEndScroll = true;
              }else{
                this.isEndScroll = false;
              }
            },
          ),
          Align(
            alignment: Alignment(0.0, 0.9),
            child: TabPageSelector(
              color: this.unSelectetColor,
              selectedColor: this.selectetedColor,
              indicatorSize: this.pageControllerSize,
              controller: _tabController,
            ),
          ),
        ],
      ),
    );
  }

  _getImagesWithUrl() {
    List<Widget> tempArr = [];
    for (var item in _tempShowPictures) {
      tempArr.add(InkWell(child: Image.network(item['imageUrl'].toString(), fit: BoxFit.fill,), onTap: () {
        callBack(item['link'].toString());
      },));
    }
    return tempArr;
  }

  _handleTimeout(Timer timer) {
    if (isEndScroll == true) {
      _index += 1;
      _pageController.animateToPage(
        _index,
        duration: Duration(milliseconds: 100),
        curve: Curves.linear,
      );
      _tabController.animateTo(_index % (_adPictures.length));
    }
  }

  void _onPageChanged(int index) {
    _index = index;
    if (index == 0) {
      _tabController.animateTo(_adPictures.length - 1);
      _pageController.jumpToPage(_adPictures.length);
    } else if (index == _adPictures.length + 1) {
      _tabController.animateTo(0);
      _pageController.jumpToPage(1);
    } else {
      _tabController.animateTo(index - 1);
    }
  }
}

class Chflutterview {
  static const MethodChannel _channel =
      const MethodChannel('chflutterview');

  static Future<String> get platformVersion async {
    final String version = await _channel.invokeMethod('getPlatformVersion');
    return version;
  }
}

